unit engine;

interface
uses dxdraws,DXinput, windows;
type
TSprite=class   // klasa podstawowa dla wszystkich obiektow
        x,y:real;
        vx,vy:real;
        sx,sy:integer; // rozmiary
        mx,my:integer; // rozmiary pola po jakim mozna sie poruszac
        deadx:integer; // linia za ktora pilke uwaza sie martwa
        points:integer; // jakies punkty
        procedure ustaw(wx,wy,wvx,wvy:real;wsx,wsy,wmx,wmy,wdeadx:integer);virtual;
        function xx:integer;virtual; // zwraca calkowita wartosc x
        function yy:integer;virtual; // zwraca calkowita wartosc y
        procedure draw(dxdraw:TDXDraw); virtual;
end;

TKontrol=(kKey,KMouse);
TPaletka=class(TSprite)
        procedure paletka(dxdraw:TDXDraw;input:TDXInput;kontroler:TKontrol);
        procedure move;
        procedure draw(dxdraw:TDXDraw); overload;
end;

TBall=class(TSprite)
        procedure ball(dxdraw:TDXDraw;var paletka:TPaletka;var who:byte);
        procedure fire; // wystrzeliwuje pilke
        procedure move(var paletka:TPaletka;var who:byte);
        procedure draw(dxdraw:TDXDraw); overload;
end;



implementation

  /////////////
 // TSPRITE //
/////////////
procedure TSprite.ustaw(wx,wy,wvx,wvy:real;wsx,wsy,wmx,wmy,wdeadx:integer);
begin
  x:=wx;
  y:=wy;
  vx:=wvx;
  vy:=wvy;
  sx:=wsx;
  sy:=wsy;
  mx:=wmx;
  my:=wmy;
  deadx:=wdeadx;
  points:=0;
end;

function TSprite.xx:integer;
begin
  xx:=trunc(x);
end;

function TSprite.yy:integer;
begin
  yy:=trunc(y);
end;

procedure TSprite.draw(dxdraw:TDXDraw);
begin
  dxdraw.surface.canvas.Rectangle(xx-sx div 2,yy-sy div 2,xx+sx div 2,yy+sy div 2);
end;

  //////////////
 // TPaletka //
//////////////

procedure TPaletka.paletka(dxdraw:TDXDraw;input:TDXInput;kontroler:TKontrol);
begin
  vx:=0;
  vy:=0;
  case kontroler of
    kKey:begin
          if input.Keyboard.Keys[vk_up]=true then vy:=-input.Keyboard.ButtonCount / 10;
          if input.Keyboard.Keys[vk_down]=true then vy:=input.Keyboard.ButtonCount / 10;
          if input.Keyboard.Keys[vk_left]=true then vx:=-input.Keyboard.ButtonCount / 10;
          if input.Keyboard.Keys[vk_right]=true then vx:=input.Keyboard.ButtonCount / 10;
        end;
    kMouse:begin
             vx:=(input.Mouse.X*x) / x;
             vy:=(input.Mouse.y*y) / y;
           end;
  end;
  move;
  draw(dxdraw);
end;

procedure TPaletka.move;
var
tx,ty:real;
begin
  tx:=x+vx;
  ty:=y+vy;
  if (ty<=0) or (ty>=my) then
  begin
    vy:=0;
    ty:=y;
  end;
  if (tx<=0) or (tx>=mx) then
  begin
    vx:=0;
    tx:=x;
  end;
  x:=tx;
  y:=ty;
end;

procedure TPaletka.draw(dxdraw:TDXDraw);
begin
  dxdraw.surface.canvas.brush.Color:=RGB(0,0,255);
  dxdraw.surface.canvas.pen.Color:=RGB(0,0,255);
  dxdraw.surface.canvas.Ellipse(xx-sx div 2,yy-sy div 2,xx+sx div 2,yy+sy div 2);
end;

  ///////////
 // TBALL //
///////////

procedure TBall.ball(dxdraw:TDXDraw;var paletka:TPaletka;var who:byte);
begin
  move(paletka,who);
  draw(dxdraw);
end;

procedure TBall.fire;
begin
  x:=deadx;
  y:=my div 2;
  vx:=-(random+5.2);
  vy:=(random - random)*2;
end;

procedure TBall.move(var paletka:TPaletka;var who:byte);
var
tx,ty:real;
x1,x2,y1,y2:real;
begin
  tx:=x+vx;
  ty:=y+vy;
  begin
    x1:=paletka.x - paletka.sx / 2 - vx;
    y1:=paletka.y - paletka.sy / 2 - vy;
    x2:=paletka.x + paletka.sx / 2 + vx;
    y2:=paletka.y + paletka.sy / 2 + vy;
  end;
  if (tx>=x1) and (ty>=y1) and (tx<=x2) and (ty<=y2) then
  begin
    tx:=x;
    vx:=-(2*vx-paletka.vx) / 2;
    vy:=-(2*vy+paletka.vy) / 2;
    tx:=tx+vx;
    inc(paletka.points);
    who:=abs(who-1);
  end;
  if (tx<=0) or (tx>=mx) then
  begin
    if (tx>=mx) and ((vx>0) or (vy>0)) then
    begin
      dec(paletka.points);
      vx:=0;
      vy:=0;
    end;
    vx:=-vx;
    tx:=tx+vx;
  end;
  if (ty<=0) or (ty>=my) then
  begin
    vy:=-vy;
    ty:=ty+vy;
  end;
  x:=tx;
  y:=ty;
end;

procedure TBall.draw(dxdraw:TDXDraw);
begin
  dxdraw.surface.canvas.brush.Color:=RGB(255,0,0);
  dxdraw.surface.canvas.pen.Color:=RGB(255,0,0);
  dxdraw.surface.canvas.Ellipse(xx-sx div 2,yy-sy div 2,xx+sx div 2,yy+sy div 2);
end;

end.
